@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,37 +20,43 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 from zope.interface import implements  # pylint: disable=unused-import
 
-from log import m18n, m18nc, logDebug
-from common import LIGHTSOURCES, Internal, isAlive, ZValues, Debug
-from common import nativeString
-from wind import Wind
 from twisted.internet.defer import succeed
 
-from qt import Qt, QMetaObject, variantValue
+from log import logDebug, id4
+from mi18n import i18n, i18nc
+from common import LIGHTSOURCES, Internal, isAlive, ZValues, Debug
+from common import StrMixin, Speeds
+from wind import Wind
+
+from qt import Qt, QMetaObject
 from qt import QGraphicsScene, QGraphicsItem, QGraphicsRectItem, QPen, QColor
 
 from dialogs import QuestionYesNo
-from guiutil import decorateWindow
+from guiutil import decorateWindow, sceneRotation
 from board import SelectorBoard, DiscardBoard
 from tileset import Tileset
 from meld import Meld
 from humanclient import HumanClient
 from uitile import UITile
 from uiwall import UIWall
-from animation import MoveImmediate, afterQueuedAnimations
+from animation import AnimationSpeed, afterQueuedAnimations
 from scoringdialog import ScoringDialog
 
 
-class FocusRect(QGraphicsRectItem):
+class FocusRect(QGraphicsRectItem, StrMixin):
 
-    """show a focusRect with blue border around focused tile or meld"""
+    """show a focusRect with blue border around focused tile or meld.
+    We can NOT do this as a child of the item or the focus rect would
+    have to stay within tile face: The adjacent tile will cover the
+    focus rect because Z order is only relevant for items having the
+    same parent"""
 
     def __init__(self):
         QGraphicsRectItem.__init__(self)
         pen = QPen(QColor(Qt.blue))
         pen.setWidth(6)
         self.setPen(pen)
-        self.setZValue(ZValues.marker)
+        self.setZValue(ZValues.markerZ)
         self._board = None
         self.hide()
 
@@ -64,7 +70,7 @@ class FocusRect(QGraphicsRectItem):
         """assign and show/hide as needed"""
         if value and not isAlive(value):
             logDebug(
-                u'assigning focusRect to a non-alive board %s/%s' %
+                'assigning focusRect to a non-alive board %s/%s' %
                 (type(value), value))
             return
         if value:
@@ -82,7 +88,7 @@ class FocusRect(QGraphicsRectItem):
         rect = board.tileFaceRect()
         rect.setWidth(rect.width() * board.focusRectWidth())
         self.setRect(rect)
-        self.setRotation(board.sceneRotation())
+        self.setRotation(sceneRotation(board))
         self.setScale(board.scale())
         if board.focusTile:
             board.focusTile.setFocus()
@@ -92,6 +98,10 @@ class FocusRect(QGraphicsRectItem):
                         and board.isEnabled() and board.hasFocus and bool(game) and not game.autoPlay)
 
 
+    def __str__(self):
+        """for debugging"""
+        return 'FocusRect({} on {})'.format(id4(self), self.board if self.board else 'NOBOARD')
+
 class SceneWithFocusRect(QGraphicsScene):
 
     """our scene with a potential Qt bug fix. FocusRect is a blue frame around a tile or meld"""
@@ -166,9 +176,10 @@ class GameScene(SceneWithFocusRect):
                     self.explainView.hide()
                 self.mainWindow.scene = None
         self.mainWindow.updateGUI()
-        self.mainWindow.adjustView()
+        self.mainWindow.adjustMainView()
 
-    def showShadowsChanged(self, dummyOldValue, dummyNewValue):
+    @afterQueuedAnimations
+    def showShadowsChanged(self, deferredResult, dummyOldValue, dummyNewValue): # pylint: disable=unused-argument
         """if the wanted shadow direction changed, apply that change now"""
         for uiTile in self.graphicsTileItems():
             uiTile.setClippingFlags()
@@ -177,8 +188,7 @@ class GameScene(SceneWithFocusRect):
     def handSelectorChanged(self, handBoard):
         """update all relevant dialogs"""
         if self.game and not self.game.finished():
-            self.game.wall.decoratePlayer(
-                handBoard.player)  # pylint:disable=no-member
+            handBoard.player.showInfo()
         # first decorate walls - that will compute player.handBoard for
         # explainView
         if self.explainView:
@@ -197,12 +207,12 @@ class GameScene(SceneWithFocusRect):
         """abort current game"""
         # to be implemented by children
 
-    def adjustView(self):
+    def adjustSceneView(self):
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
         if self.game:
-            with MoveImmediate():
-                self.game.wall.decorate()
+            with AnimationSpeed():
+                self.game.wall.decorate4()
                 for uiTile in self.game.wall.tiles:
                     if uiTile.board:
                         uiTile.board.placeTile(uiTile)
@@ -211,15 +221,17 @@ class GameScene(SceneWithFocusRect):
         """apply preferences"""
         self.mainWindow.actionAngle.setEnabled(
             bool(self.game) and Internal.Preferences.showShadows)
-        with MoveImmediate():
+        with AnimationSpeed():
             for item in self.nonTiles():
-                item.tileset = Tileset.activeTileset()
+                if hasattr(item, 'tileset'):
+                    item.tileset = Tileset.current()
 
     def prepareHand(self):
         """redecorate wall"""
         self.mainWindow.updateGUI()
         if self.game:
-            self.game.wall.decorate()
+            with AnimationSpeed(Speeds.windMarker):
+                self.game.wall.decorate4()
 
     def updateSceneGUI(self):
         """update some actions, all auxiliary windows and the statusbar"""
@@ -244,7 +256,7 @@ class GameScene(SceneWithFocusRect):
         """change the lightSource"""
         self.game.wall.lightSource = self.newLightSource()
         self.focusRect.refresh()
-        self.mainWindow.adjustView()
+        self.mainWindow.adjustMainView()
 
     def __showBalance(self):
         """show the player balances in the status bar"""
@@ -274,6 +286,8 @@ class GameScene(SceneWithFocusRect):
         """remove all tiles from scene"""
         for item in self.graphicsTileItems():
             self.removeItem(item)
+        for wind in Wind.all:
+            wind.marker = None
         self.focusRect.hide()
 
 
@@ -338,7 +352,7 @@ class PlayingScene(GameScene):
         self.discardBoard = DiscardBoard()
         self.addItem(self.discardBoard)
 
-        self.adjustView()
+        self.adjustSceneView()
 
     def showWall(self):
         """shows the wall according to the game rules (length may vary)"""
@@ -362,7 +376,7 @@ class PlayingScene(GameScene):
             self.game = None
             return succeed(True)
         else:
-            return QuestionYesNo(m18n("Do you really want to abort this game?"), always=True).addCallback(
+            return QuestionYesNo(i18n("Do you really want to abort this game?"), always=True).addCallback(
                 gotAnswer, autoPlaying)
 
     def keyPressEvent(self, event):
@@ -373,13 +387,13 @@ class PlayingScene(GameScene):
                 self.clientDialog.keyPressEvent(event)
         GameScene.keyPressEvent(self, event)
 
-    def adjustView(self):
+    def adjustSceneView(self):
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
         if self.game:
-            with MoveImmediate():
+            with AnimationSpeed():
                 self.discardBoard.maximize()
-        GameScene.adjustView(self)
+        GameScene.adjustSceneView(self)
 
     @property
     def startingGame(self):
@@ -420,8 +434,8 @@ class PlayingScene(GameScene):
         if not game:
             connections = list(
                 x.connection for x in HumanClient.humanClients if x.connection)
-            title = u', '.join(u'{name}/{url}'.format(name=x.username, url=x.url)
-                               for x in connections)
+            title = ', '.join('{name}/{url}'.format(name=x.username, url=x.url)
+                              for x in connections)
             if title:
                 decorateWindow(mainWindow, title)
         else:
@@ -491,16 +505,16 @@ class ScoringScene(GameScene):
                 self.game = None
             return result
         if Debug.quit:
-            logDebug(u'ScoringScene.abort invoked')
+            logDebug('ScoringScene.abort invoked')
         if not self.game:
             return succeed(True)
         elif self.game.finished():
             self.game = None
             return succeed(True)
         else:
-            return QuestionYesNo(m18n("Do you really want to abort this game?"), always=True).addCallback(answered)
+            return QuestionYesNo(i18n("Do you really want to abort this game?"), always=True).addCallback(answered)
 
-    def __moveTile(self, uiTile, wind, lowerHalf):
+    def __moveTile(self, uiTile, wind, toConcealed):
         """the user pressed a wind letter or X for center, wanting to move a uiTile there"""
         # this tells the receiving board that this is keyboard, not mouse navigation>
         # needed for useful placement of the popup menu
@@ -509,11 +523,12 @@ class ScoringScene(GameScene):
             receiver = self.selectorBoard
         else:
             receiver = self.game.players[wind].handBoard
-        if receiver != currentBoard or bool(lowerHalf) != bool(uiTile.yoffset):
-            movingLastMeld = uiTile.tile in self.computeLastMeld()
+        movingMeld = currentBoard.uiMeldWithTile(uiTile)
+        if receiver != currentBoard or toConcealed != movingMeld.meld.isConcealed:
+            movingLastMeld = movingMeld.meld == self.computeLastMeld()
             if movingLastMeld:
                 self.scoringDialog.clearLastTileCombo()
-            receiver.dropTile(uiTile, lowerHalf)
+            receiver.dropTile(uiTile, toConcealed)
             if movingLastMeld and receiver == currentBoard:
                 self.scoringDialog.fillLastTileCombo()
 
@@ -523,7 +538,7 @@ class ScoringScene(GameScene):
         key = event.key()
         wind = chr(key % 128)
         windsX = ''.join(x.char for x in Wind.all)
-        moveCommands = m18nc('kajongg:keyboard commands for moving tiles to the players '
+        moveCommands = i18nc('kajongg:keyboard commands for moving tiles to the players '
                              'with wind ESWN or to the central tile selector (X)', windsX)
         uiTile = self.focusItem()
         if wind in moveCommands:
@@ -552,13 +567,13 @@ class ScoringScene(GameScene):
                     return
         GameScene.keyPressEvent(self, event)
 
-    def adjustView(self):
+    def adjustSceneView(self):
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
         if self.game:
-            with MoveImmediate():
+            with AnimationSpeed():
                 self.selectorBoard.maximize()
-        GameScene.adjustView(self)
+        GameScene.adjustSceneView(self)
 
     def prepareHand(self):
         """redecorate wall"""
@@ -597,6 +612,5 @@ class ScoringScene(GameScene):
             cbLastMeld = self.scoringDialog.cbLastMeld
             idx = cbLastMeld.currentIndex()
             if idx >= 0:
-                return Meld(nativeString(
-                    variantValue(cbLastMeld.itemData(idx))))
+                return Meld(cbLastMeld.itemData(idx))
         return Meld()
